const { bail, info, error } = require('@cloudflare/cli-helpers').setup({
  cliName: 'memsource-cli'
});
const { getTranslationsMap, getNumOfLinks, getLocales } = require('./utils');

/**
 * getLinkErrors
 * Checks for valid markdown link syntax given a JSON file.
 *
 * @param {Array} [{String} filename, {Object} textInFile]
 * @return {Array} errors found
 */
const getLinkErrors = ([filename, textInFile]) => {
  let errors = [];
  let [leftSquare, rightSquare, leftParen, rightParen] = [0, 0, 0, 0];
  Object.keys(textInFile).forEach(t => {
    const markdown = textInFile[t];
    if (typeof markdown === 'string') {
      // create a mapped array of the number of occurences of each type of
      // bracket in the markdown
      [leftSquare, rightSquare, leftParen, rightParen] = [
        '\\[',
        '\\]',
        '\\(',
        '\\)'
      ].map(b => (markdown.match(new RegExp(b, 'g')) || []).length);
      if (leftSquare !== rightSquare || leftParen !== rightParen) {
        // markdown such as 'A) ...' is accepted
        if (markdown[1] !== ')') {
          errors.push({
            errorName: 'Missing Brackets or Parentheses',
            filename,
            translationKey: t,
            markdown
          });
        }
      } else if (
        (leftSquare > 0 || rightSquare > 0) &&
        (leftParen > 0 || rightParen > 0)
      ) {
        // there are brackets but the regex can't find any links
        if (getNumOfLinks(markdown) === 0) {
          errors.push({
            errorName: 'Invalid Markdown Link Syntax',
            filename,
            translationKey: t,
            markdown
          });
        }
      }
    }
  });

  return errors;
};

/**
 * showLinkErrors
 * Gets the translations as JSON objects from every file
 * for each locale and calls getLinkErrors. If there are
 * any errors in the file, the error is logged.
 */
const showLinkErrors = () => {
  getLocales().forEach(locale => {
    const translations = getTranslationsMap(locale);
    for (const fileObj of translations) {
      const errors = getLinkErrors(fileObj);
      if (errors.length) {
        errors.forEach(({ errorName, filename, translationKey, markdown }) => {
          error(errorName.toUpperCase());
          info(`${filename} | ${translationKey}`);
          info(`\n${markdown}\n`);
        });
        bail('There are translation errors.');
      }
    }
  });
};

module.exports = {
  showLinkErrors,
  getLinkErrors
};
