const findConfig = require('find-config');
const axios = require('axios');
const {
  authenticate,
  baseUrl,
  getSplitTranslations,
  getTranslationPackageRoot,
  localeMap,
  writeFormattedJsonFile,
  validStatus
} = require('./utils');
const config = findConfig.require('memsource.config.js');
const { MEMSOURCE_PROJECT } = config;
const { info, success, bail } = require('@cloudflare/cli-helpers').setup({
  cliName: 'memsource-cli'
});

function syncCompletedJobs(project = MEMSOURCE_PROJECT, token) {
  return axios
    .get(`${baseUrl}projects/${project}`, { params: { token } })
    .then(async res => {
      const fileCount = res.data.progress.totalCount;
      const finishedCount = res.data.progress.finishedCount;

      // We can only resolve one finished translation job at a time.
      // If we have multiple completed jobs, it can lead to conflicts.
      // So we bail if there is more than one
      if (finishedCount > 1) {
        bail(
          `Found ${res.data.progress.finishedCount} files in ${validStatus} state. There should only ever be one file. Refusing to download.`
        );
      }

      if (finishedCount === 0) {
        info(`No files in ${validStatus} state. Nothing to do.`);
        process.exit(0);
      }

      // Since the response is paginated, we go through each page and look for DELIVERED files
      const reqs = [];
      let pageNumber = 0;
      let maxPage = Math.ceil(fileCount / 50);
      let workflowLevelsResponse = await axios.get(
        `${baseUrl}projects/${project}/workflowSteps`,
        {
          params: { token }
        }
      );
      const workflowLevels = workflowLevelsResponse.data.projectWorkflowSteps.map(
        currentValue => currentValue.workflowLevel
      );

      // Only get the files that are in validStatus i.e. DELIVERED
      while (pageNumber < maxPage) {
        // Loop through each workflow step to make sure we get a delivered state file in any workflow step
        workflowLevels.forEach(workflowLevel => {
          reqs.push(
            axios.get(`${baseUrl}projects/${project}/jobs`, {
              params: {
                token,
                pageNumber,
                workflowLevel,
                status: validStatus
              }
            })
          );
        });
        pageNumber++;
      }

      return Promise.all(reqs);
    })
    .then(results => {
      const reqs = [];
      results.forEach(({ data }) => {
        data.content.forEach(job => {
          const { filename, targetLang, status } = job;

          if (status === validStatus) {
            reqs.push(
              axios
                .get(
                  `${baseUrl}projects/${project}/jobs/${job.uid}/targetFile`,
                  {
                    params: { token }
                  }
                )
                .then(jobData => {
                  const phrases = jobData.data;
                  info(
                    `Job completed for ${filename}, in target locale: ${targetLang}`
                  );

                  return {
                    filename,
                    targetLang,
                    status,
                    phrases
                  };
                })
            );
          } else {
            info(`File not in ${validStatus} state. Nothing to do.`);
          }
        });
      });

      return Promise.all(reqs);
    });
}

const down = async () => {
  const response = await authenticate();
  const token = response.data.token;
  const jobs = await syncCompletedJobs(MEMSOURCE_PROJECT, token);

  const targetLangOutputPath = `${getTranslationPackageRoot()}/src/locale`;

  jobs.forEach(({ targetLang, phrases }) => {
    getSplitTranslations(phrases).forEach(([filename, phrases]) => {
      writeFormattedJsonFile(
        `${targetLangOutputPath}/${localeMap[targetLang]}/${filename}`,
        phrases
      );
      success('Writing file ' + filename);
    });
  });
};

module.exports = down;
